# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winrt._winrt_windows_phone_system_power import (
    PowerManager,
)

__all__ = [
    "PowerSavingMode",
    "PowerManager",
]

class PowerSavingMode(enum.IntEnum):
    OFF = 0
    ON = 1

